@@ -24,8 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gdk.h"
-#include "config.h"
 
 #include "gdkwindow.h"
 #include "gdkinputprivate.h"
@@ -188,6 +188,9 @@ gdk_window_new (GdkWindow     *parent,
 
   private->parent = parent_private;
 
+  private->accept_focus = TRUE;
+  private->focus_on_map = TRUE;
+
   if (attributes_mask & GDK_WA_X)
     x = attributes->x;
   else
@@ -265,6 +268,9 @@ gdk_window_new (GdkWindow     *parent,
   if (parent_private)
     parent_private->children = g_list_prepend (parent_private->children, window);
   
+  if (attributes_mask & GDK_WA_TYPE_HINT)
+    gdk_window_set_type_hint (window, attributes->type_hint);
+
   return window;
 }
 
@@ -1438,8 +1444,8 @@ gdk_window_set_transient_for (GdkWindow *window,
 }
 
 void
-gdk_window_set_background (GdkWindow *window,
-			   GdkColor  *color)
+gdk_window_set_background (GdkWindow      *window,
+			   const GdkColor *color)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   
@@ -1470,6 +1476,12 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   g_return_if_fail (pixmap == NULL || !parent_relative);
   g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
 
+  if (pixmap && !gdk_drawable_get_colormap (pixmap))
+    {
+      g_warning ("gdk_window_set_back_pixmap(): pixmap must have a colormap");
+      return;
+    }
+  
   old_pixmap = private->bg_pixmap;
 
   if (parent_relative)
@@ -1990,6 +2002,38 @@ gdk_window_set_override_redirect (GdkWindow *window,
   /* N/A */
 }
 
+void
+gdk_window_set_accept_focus (GdkWindow *window,
+			     gboolean accept_focus)
+{
+  GdkWindowObject *private;
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+  
+  accept_focus = accept_focus != FALSE;
+
+  if (private->accept_focus != accept_focus)
+    private->accept_focus = accept_focus;
+}
+
+void
+gdk_window_set_focus_on_map (GdkWindow *window,
+			     gboolean focus_on_map)
+{
+  GdkWindowObject *private;
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+  
+  focus_on_map = focus_on_map != FALSE;
+
+  if (private->focus_on_map != focus_on_map)
+    private->focus_on_map = focus_on_map;
+}
+
 void          
 gdk_window_set_icon (GdkWindow *window, 
 		     GdkWindow *icon_window,
@@ -2308,7 +2352,7 @@ GdkWindow*
 gdk_window_foreign_new_for_display (GdkDisplay      *display,
 				    GdkNativeWindow  anid)
 {
-  return (GdkWindow*) gdk_drawable_ref (anid);
+  return (GdkWindow*) gdk_drawable_ref ((GdkDrawable *)anid);
 }
 
 GdkWindow*
@@ -2324,3 +2368,13 @@ gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
 
   return (GdkWindow*) (anid);
 }
+
+void
+gdk_window_enable_synchronized_configure (GdkWindow *window)
+{
+}
+
+void
+gdk_window_configure_finished (GdkWindow *window)
+{
+}
