@@ -732,7 +732,7 @@ gtk_list_store_set_valist_internal (GtkListStore *list_store,
       GValue value = { 0, };
       gchar *error = NULL;
 
-      if (column >= list_store->n_columns)
+      if (column < 0 || column >= list_store->n_columns)
 	{
 	  g_warning ("%s: Invalid column number %d added to iter (remember to end your list of columns with a -1)", G_STRLOC, column);
 	  break;
@@ -1789,7 +1789,7 @@ gtk_list_store_has_default_sort_func (GtkTreeSortable *sortable)
  * has the same effect as calling 
  * <informalexample><programlisting>
  * gtk_list_store_insert (list_store, iter, position);
- * gtk_list_store_set (list_store_iter, ...);
+ * gtk_list_store_set (list_store, iter, ...);
  * </programlisting></informalexample>
  * with the difference that the former will only emit a row_inserted signal,
  * while the latter will emit row_inserted, row_changed and, if the list store
